#include "main.h"

/* local prototypes */

/********************

  Read_Help_File

*********************/

ReturnType Read_Help_File()
{
  FILE *HelpFile;
  struct stat HelpFileStat;
  char Buffer[MY_BUFSIZ+1];

  if ((HelpFile = fopen(HELP_FILE,"r")) == NULL)
    {
      sprintf(Buffer,"File Error:\n\nCould not open help file %s",HELP_FILE);
      ErrorMessage(Buffer);
      return (RETURN_NOTOK);
    }
  if (stat(HELP_FILE,&HelpFileStat) != 0)
    {
      sprintf(Buffer,"File Error:\n\nCould not stat help file %s",HELP_FILE);
      ErrorMessage(Buffer);
      return (RETURN_NOTOK);
    }
  G_HelpString = (char *)malloc(HelpFileStat.st_size+1);
  read(fileno(HelpFile),G_HelpString,HelpFileStat.st_size-1);
  G_HelpString[HelpFileStat.st_size+1] = '\0';
  G_HelpString[HelpFileStat.st_size] = '\0';
  fclose(HelpFile);
  return (RETURN_OK);
}

/***********************

  Read_Calendar -- Read calendar into memory


************************/


ReturnType Read_Calendar(char *Filename)
{
  FILE *CalendarFile;
  char  Line[MY_BUFSIZ+1];
  struct Calendar_Struct DateEntry;
  struct Appointment_Struct AppointmentEntry;
  int lcv = 0;
  char  Buffer[MY_BUFSIZ+1];

  if ((CalendarFile = fopen(Filename,"r")) == NULL)
    {
      sprintf(Buffer,"File Error:\n\nCould not open file %s for reading.\n",Filename);
      ErrorMessage(Buffer);
      G_CalendarList = NULL;
      G_CurrentEntryInCalendarList = NULL;
      G_CurrentNumInCalendarList = 0;
      G_NumInCalendarList = 0;
      return(RETURN_NOTOK);
    }
  fgets(Line,MY_BUFSIZ,CalendarFile);
  if (Line != NULL)
    {
      Remove_CR(Line);
      if (strcmp(Line,"#CS1.0") != 0)
        {
          sprintf(Buffer,"File Error:\n\n%s is not a XmCalendar file.\n",Filename);
	  ErrorMessage(Buffer);
	  G_CalendarList = NULL;
	  G_CurrentEntryInCalendarList = NULL;
	  G_CurrentNumInCalendarList = 0;
	  G_NumInCalendarList = 0;
	  return(RETURN_NOTOK);
        }
      if (!feof(CalendarFile)) fgets(Line,MY_BUFSIZ,CalendarFile);
    }
  while (!feof(CalendarFile))
    {
      Remove_CR(Line);
      DateEntry.Year = atoi(Line);

      fgets(Line,MY_BUFSIZ,CalendarFile);
      Remove_CR(Line);
      DateEntry.Month = atoi(Line);

      fgets(Line,MY_BUFSIZ,CalendarFile);
      Remove_CR(Line);
      DateEntry.Day = atoi(Line);

      fgets(Line,MY_BUFSIZ,CalendarFile);
      Remove_CR(Line);
      DateEntry.NumberOfAppointments = atoi(Line);

/*       Add_Date(DateEntry); */

      lcv = 0;
      while (!feof(CalendarFile) && (lcv != DateEntry.NumberOfAppointments))
	{
	  lcv++;
/* 	  fprintf(stderr,"read appointment %d\n",lcv); */

	  fgets(Line,MY_BUFSIZ,CalendarFile);
	  Remove_CR(Line);
	  AppointmentEntry.Hour = atoi(Line);
	  
	  fgets(Line,MY_BUFSIZ,CalendarFile);
	  Remove_CR(Line);
	  AppointmentEntry.Minutes = atoi(Line);

	  fgets(Line,MY_BUFSIZ,CalendarFile);
	  Remove_CR(Line);
	  if (strcmp(Line,"TRUE") == 0)
	    {
	      AppointmentEntry.ReminderAlarm  = True;
	    }
	  else {
	    AppointmentEntry.ReminderAlarm  = False;
	  }

	  fgets(Line,MY_BUFSIZ,CalendarFile);
	  Remove_CR(Line);
	  AppointmentEntry.ReminderWarning = atoi(Line);

	  fgets(Line,MY_BUFSIZ,CalendarFile);
	  Remove_CR(Line);
	  AppointmentEntry.Repeat = atoi(Line);
	  
	  fgets(Line,MY_BUFSIZ,CalendarFile);
	  Remove_CR(Line);
	  AppointmentEntry.RepeatForNumDays = atoi(Line);

	  fgets(Line,MY_BUFSIZ,CalendarFile);
	  AppointmentEntry.Message = strdup(Line);

	  fgets(Line,MY_BUFSIZ,CalendarFile);
	  while (!feof(CalendarFile) && (strcmp(Line,"#ME\n") != 0))
	    {
	      AppointmentEntry.Message = (char *)realloc(AppointmentEntry.Message,sizeof(Line));
	      strcat(AppointmentEntry.Message,Line);
	      fgets(Line,MY_BUFSIZ,CalendarFile);
	    }
	  AppointmentEntry.ListPosition = 0;
 	  Add_Appointment(DateEntry,AppointmentEntry); 

	  free(AppointmentEntry.Message);
	}
      if (lcv != DateEntry.NumberOfAppointments)
	{
	  sprintf(Buffer,"File Error:\n\nNumber Of Appointments != number actually read for date %d/%d/%d\nThis should not happen!\n",
		  DateEntry.Month,DateEntry.Day,DateEntry.Year);
	  ErrorMessage(Buffer);
	  return(RETURN_NOTOK);
	}
      
      if (!feof(CalendarFile)) fgets(Line,MY_BUFSIZ,CalendarFile);
    }
  fclose (CalendarFile);
  return(RETURN_OK);
}


/***********************

  Write_Calendar -- Write calendar to disk


************************/


ReturnType Write_Calendar(char *Filename)
{
  FILE *CalendarFile;
  char  Buffer[MY_BUFSIZ+1];
  Calendar_Type *Temp;
  Appointment_Type *ATemp;
  
  if (G_CalendarList == NULL)
    {
      ErrorMessage("File Error: Calendar Empty, nothing to save.");
      return(RETURN_OK);
    }
  if ((CalendarFile = fopen(Filename,"w")) == NULL)
    {
      sprintf(Buffer,"File Error:\n\nCould not open file %s for writing.\n",Filename);
      ErrorMessage(Buffer);
      return(RETURN_NOTOK);
    }
  fprintf(CalendarFile,"#CS1.0\n");
/*   fprintf(stderr,"write filename: %s\n",Filename); */
  
  Temp = G_CalendarList;
  while (Temp != NULL)
    {
      fprintf(CalendarFile,"%d\n",Temp->Year);
      fprintf(CalendarFile,"%d\n",Temp->Month);
      fprintf(CalendarFile,"%d\n",Temp->Day);
      fprintf(CalendarFile,"%d\n",Temp->NumberOfAppointments);
      ATemp = Temp->Appointments;
      while (ATemp != NULL)
	{
	  fprintf(CalendarFile,"%d\n",ATemp->Hour);
	  fprintf(CalendarFile,"%d\n",ATemp->Minutes);

	  if (ATemp->ReminderAlarm == True)
	    {
	      fprintf(CalendarFile,"TRUE\n");
	    }
	  else {
	    fprintf(CalendarFile,"FALSE\n");
	  }
	  
	  fprintf(CalendarFile,"%d\n",ATemp->ReminderWarning);
	  fprintf(CalendarFile,"%d\n",ATemp->Repeat);
	  fprintf(CalendarFile,"%d\n",ATemp->RepeatForNumDays);
	  fprintf(CalendarFile,"%s\n#ME\n",ATemp->Message);
	  
	  ATemp = ATemp->Next;
	}
      Temp = Temp->Next;
    }
  fclose (CalendarFile);
  G_ChangedNotSaved = False;
  return(RETURN_OK);
}

