#define MAIN
#include "main.h"
#include "menu.h"
#include "help.h"


/* function prototypes */
void ToggleJulianDates();
void PreviousMonth_Calendar();
void NextMonth_Calendar();
void Print_Usage(char *ProgramName);
void Create_Calendar();
void Show_Today();
void Set_Today();
void DisplayMonth_Calendar(int Month,
			   int Year);
void DisplayDaily_Appointments(Widget   w,
			       int     Day);
void Create_ButtonBar();
void  Help_Line(Widget   w,
		char     *Key);


/* ButtonBar Widgets */
Widget W_ButtonBar_Frame;

/* RoloForm Widgets */
Widget W_Calendar_Frame;
Widget W_Calendar_RC;
Widget W_Help_Label;
Widget W_Days_Button[42];
Widget W_Month_Label;
Widget W_Year_Label;
Widget W_Filename_Text;

void main(int argc,
	  char **argv)
{
  int lcv;
  char *UserName = NULL;
  struct passwd *PasswdEntry;
  Bool AlternateUserFile = False;

  Set_Today();
  G_Current_Month = G_Today_Month;
  G_Current_Year = G_Today_Year;
  G_Current_Day = G_Today_Day;
  if (argc >= 1)
    {
      for (lcv=0;lcv < argc; lcv++)
	{
	  if ((strcmp(argv[lcv],"-help") == 0) || (strcmp(argv[lcv],"-h") == 0))
	    {
	      G_ASCII_Mode = True;
	      Print_Usage(argv[0]);
	      exit();
	    }
	  if ((strcmp(argv[lcv],"-user") == 0) || (strcmp(argv[lcv],"-u") == 0))
	    {
	      lcv++;
	      if (argc != lcv)
		{
		  UserName = strdup(argv[lcv]);
		  AlternateUserFile = True;
		}
	    }
	}
    }
  
  if (UserName == NULL)
    {
      if ((PasswdEntry = getpwuid(getuid())) == NULL)
	{
	  perror("getpwuid");
	  exit();
	}
      UserName = strdup(PasswdEntry->pw_name);
    }
  G_Calendar_Filename = (char *)malloc(MY_BUFSIZ);
  if (AlternateUserFile == True)
    {
      sprintf(G_Calendar_Filename,"%s/%s",UserName,DEFAULT_CALENDAR_FILENAME);
    }
  else {
    sprintf(G_Calendar_Filename,"%s/%s",getenv("HOME"),
	      DEFAULT_CALENDAR_FILENAME);
  }
  
  if ((G_PrinterName = (char *)getenv("PRINTER")) == NULL)
    {
      G_PrinterName = (char *)getenv("LASER");
    }
  
  W_TopLevel = XtVaAppInitialize(&G_ApplicationContext, "XmCalendar", NULL, 0,
				 &argc, argv, NULL, NULL);
  
  
  XtVaSetValues(W_TopLevel,  
		XmNtitle, PROGRAM_TITLE,
		NULL,0);
  
  W_Main_Frame = XtVaCreateManagedWidget("W_Main_Frame",
					 xmFrameWidgetClass, 
					 W_TopLevel,
					 NULL,0);
  
  W_Main_Form = XtVaCreateManagedWidget("W_Main_Form",
					xmFormWidgetClass, 
					W_Main_Frame,
					NULL,0);

  W_MenuBar = XmCreateMenuBar(W_Main_Form, "W_MenuBar", NULL, 0);
  BuildPulldownMenu(W_MenuBar, "File", 'F', File_Menu,False);
  BuildPulldownMenu(W_MenuBar, "Calendar", 'C', Calendar_Menu,False);
  BuildPulldownMenu(W_MenuBar, "Help", 'H', Help_Menu,True);
  XtVaSetValues(W_MenuBar,
		XmNrightAttachment,XmATTACH_FORM,
		XmNleftAttachment,XmATTACH_FORM,
		XmNtopAttachment,XmATTACH_FORM,
		NULL,0);
  XtManageChild(W_MenuBar);
  
  Create_ButtonBar();
  Create_Calendar(); 
  if (G_CalendarList != NULL)
    {
      G_CurrentNumInCalendarList = 1;
      G_CurrentEntryInCalendarList = G_CalendarList;
    }
  DisplayMonth_Calendar(G_Current_Month,G_Current_Year);

  XtRealizeWidget(W_TopLevel);

  Read_Help_File();
  Read_Calendar(G_Calendar_Filename); 

  SetAlarms();
  XtAppMainLoop(G_ApplicationContext);
}
 

/**********************

  Create_Calendar -- Draw the calendar form on the screen

***********************/

void Create_Calendar()
{
  XmString LabelString;
  Widget W_Calendar_Form;
  Widget W_Calendar_Form2;
  Widget W_Calendar_Separator;
  Widget W_Filename_Label;
  Widget W_Sunday_Label;
  Widget W_Monday_Label;
  Widget W_Tuesday_Label;
  Widget W_Wednesday_Label;
  Widget W_Thursday_Label;
  Widget W_Friday_Label;
  Widget W_Saturday_Label;
  int lcv,position,row;
  char Day[4];
  XmTextScanType TextSelection_Array[1];

  W_Calendar_Frame = XtVaCreateManagedWidget("W_Calendar_Frame",
					      xmFrameWidgetClass, 
					      W_Main_Form,
					      XmNleftAttachment, XmATTACH_FORM,
					      XmNrightAttachment, XmATTACH_FORM,
					      XmNbottomAttachment, XmATTACH_FORM,
					      XmNtopAttachment, XmATTACH_WIDGET,
					      XmNtopWidget, W_ButtonBar_Frame,
					      NULL,0);

  W_Calendar_Form = XtVaCreateManagedWidget("W_Calendar_Form",
					    xmFormWidgetClass, 
					    W_Calendar_Frame,
					    NULL,0);
  
  LabelString = XmStringCreate("Current Calendar Filename: ", XmSTRING_DEFAULT_CHARSET),
  W_Filename_Label= XtVaCreateManagedWidget("W_Filename_Label",
					    xmLabelWidgetClass,
					    W_Calendar_Form,
					    XmNlabelString,LabelString,
					    XmNleftAttachment, XmATTACH_FORM,
					    XmNtopAttachment, XmATTACH_FORM,
					    XmNtopOffset, 5,
					    XmNalignment, XmALIGNMENT_BEGINNING,
					    XmNrecomputeSize, False,
					    NULL,0);
  XmStringFree(LabelString);
  W_Filename_Text= XtVaCreateManagedWidget("W_Filename_Text",
					   xmTextFieldWidgetClass,
					   W_Calendar_Form,
					   XmNlabelString,LabelString,
					   XmNleftAttachment, XmATTACH_WIDGET,
					   XmNleftWidget, W_Filename_Label,
					   XmNrightAttachment, XmATTACH_FORM,
					   XmNtopAttachment, XmATTACH_FORM,
					   XmNalignment, XmALIGNMENT_BEGINNING,
					   XmNeditable, False,
					   NULL,0);

  Set_Calendar_Filename();

  W_Calendar_Separator = XtVaCreateManagedWidget("W_Calendar_Separator",
						 xmSeparatorWidgetClass, 
						 W_Calendar_Form,
						 XmNleftAttachment, XmATTACH_FORM,
						 XmNrightAttachment, XmATTACH_FORM,
						 XmNtopAttachment, XmATTACH_WIDGET,
 						 XmNtopWidget, W_Filename_Text,
						 XmNorientation, XmHORIZONTAL,
						 XmNtopOffset, 5,
						 NULL, 0);


  LabelString = XmStringCreate("Month", XmSTRING_DEFAULT_CHARSET),
  W_Month_Label= XtVaCreateManagedWidget("W_Month_Label",
					 xmLabelWidgetClass,
					 W_Calendar_Form,
					 XmNlabelString,LabelString,
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_POSITION,
					 XmNrightPosition, 40,
					 XmNtopAttachment, XmATTACH_WIDGET,
					 XmNtopWidget, W_Calendar_Separator,
					 XmNtopOffset, 10,
					 XmNalignment, XmALIGNMENT_END,
					 NULL,0);
  XmStringFree(LabelString);
  
  LabelString = XmStringCreate("Year", XmSTRING_DEFAULT_CHARSET),
  W_Year_Label= XtVaCreateManagedWidget("W_Year_Label",
					xmLabelWidgetClass,
					W_Calendar_Form,
					XmNlabelString,LabelString,
					XmNrightAttachment, XmATTACH_FORM,
					XmNleftAttachment, XmATTACH_POSITION,
					XmNleftPosition, 60,
					XmNtopAttachment, XmATTACH_WIDGET,
					XmNtopWidget, W_Calendar_Separator,
					XmNtopOffset, 10,
					XmNalignment, XmALIGNMENT_BEGINNING,
					NULL,0);
  XmStringFree(LabelString);
  
  W_Calendar_Form2 = XtVaCreateManagedWidget("W_Calendar_Form",
					     xmFormWidgetClass, 
					     W_Calendar_Form,
					     XmNleftAttachment, XmATTACH_FORM,
					     XmNrightAttachment, XmATTACH_FORM,
					     XmNtopAttachment, XmATTACH_WIDGET,
					     XmNtopWidget, W_Month_Label,
					     XmNorientation, XmHORIZONTAL,
					     XmNfractionBase, 8,
					     XmNleftOffset, 10,
					     NULL,0);

  LabelString = XmStringCreate("Sunday", XmSTRING_DEFAULT_CHARSET),
  W_Sunday_Label= XtVaCreateManagedWidget("W_Sunday_Label",
					  xmLabelWidgetClass,
					  W_Calendar_Form2,
					  XmNlabelString,LabelString,
					  XmNleftAttachment, XmATTACH_POSITION,
					  XmNleftPosition, 0,
					  XmNrightAttachment, XmATTACH_POSITION,
					  XmNrightPosition, 1,
					  XmNtopAttachment, XmATTACH_FORM,
					  XmNalignment, XmALIGNMENT_BEGINNING,
					  XmNtopAttachment, XmATTACH_POSITION,
					  XmNtopPosition, 0,
					  XmNbottomAttachment, XmATTACH_POSITION,
					  XmNbottomPosition, 1,
					  NULL,0);
  XmStringFree(LabelString);

  LabelString = XmStringCreate("Monday", XmSTRING_DEFAULT_CHARSET),
  W_Monday_Label= XtVaCreateManagedWidget("W_Monday_Label",
					  xmLabelWidgetClass,
					  W_Calendar_Form2,
					  XmNlabelString,LabelString,
					  XmNalignment, XmALIGNMENT_BEGINNING,
					  XmNleftAttachment, XmATTACH_POSITION,
					  XmNleftPosition, 1,
					  XmNrightAttachment, XmATTACH_POSITION,
					  XmNrightPosition, 2,
					  XmNtopAttachment, XmATTACH_POSITION,
					  XmNtopPosition, 0,
					  XmNbottomAttachment, XmATTACH_POSITION,
					  XmNbottomPosition, 1,
					  NULL,0);
  XmStringFree(LabelString);

  LabelString = XmStringCreate("Tuesday", XmSTRING_DEFAULT_CHARSET),
  W_Tuesday_Label= XtVaCreateManagedWidget("W_Tuesday_Label",
					   xmLabelWidgetClass,
					   W_Calendar_Form2,
					   XmNlabelString,LabelString,
					   XmNalignment, XmALIGNMENT_BEGINNING,
					   XmNleftAttachment, XmATTACH_POSITION,
					   XmNleftPosition, 2,
					   XmNrightAttachment, XmATTACH_POSITION,
					   XmNrightPosition, 3,
					   XmNtopAttachment, XmATTACH_POSITION,
					   XmNtopPosition, 0,
					   XmNbottomAttachment, XmATTACH_POSITION,
					   XmNbottomPosition, 1,
					   NULL,0);
  XmStringFree(LabelString);
  LabelString = XmStringCreate("Wednesday", XmSTRING_DEFAULT_CHARSET),
  W_Wednesday_Label= XtVaCreateManagedWidget("W_Wednesday_Label",
					     xmLabelWidgetClass,
					     W_Calendar_Form2,
					     XmNlabelString,LabelString,
					     XmNalignment, XmALIGNMENT_BEGINNING,
					     XmNleftAttachment, XmATTACH_POSITION,
					     XmNleftPosition, 3,
					     XmNrightAttachment, XmATTACH_POSITION,
					     XmNrightPosition, 4,
					     XmNtopAttachment, XmATTACH_POSITION,
					     XmNtopPosition, 0,
					     XmNbottomAttachment, XmATTACH_POSITION,
					     XmNbottomPosition, 1,
					     NULL,0);
  XmStringFree(LabelString);

  LabelString = XmStringCreate("Thursday", XmSTRING_DEFAULT_CHARSET),
  W_Thursday_Label= XtVaCreateManagedWidget("W_Thursday_Label",
					    xmLabelWidgetClass,
					    W_Calendar_Form2,
					    XmNlabelString,LabelString,
					    XmNalignment, XmALIGNMENT_BEGINNING,
					    XmNleftAttachment, XmATTACH_POSITION,
					    XmNleftPosition, 4,
					    XmNrightAttachment, XmATTACH_POSITION,
					    XmNrightPosition, 5,
					    XmNtopAttachment, XmATTACH_POSITION,
					    XmNtopPosition, 0,
					    XmNbottomAttachment, XmATTACH_POSITION,
					    XmNbottomPosition, 1,
					    NULL,0);
  XmStringFree(LabelString);
  
  LabelString = XmStringCreate("Friday", XmSTRING_DEFAULT_CHARSET),
  W_Friday_Label= XtVaCreateManagedWidget("W_Friday_Label",
					  xmLabelWidgetClass,
					  W_Calendar_Form2,
					  XmNlabelString,LabelString,
					  XmNalignment, XmALIGNMENT_BEGINNING,
					  XmNleftAttachment, XmATTACH_POSITION,
					  XmNleftPosition, 5,
					  XmNrightAttachment, XmATTACH_POSITION,
					  XmNrightPosition, 6,
					  XmNtopAttachment, XmATTACH_POSITION,
					  XmNtopPosition, 0,
					  XmNbottomAttachment, XmATTACH_POSITION,
					  XmNbottomPosition, 1,
					  NULL,0);
  XmStringFree(LabelString);

  LabelString = XmStringCreate("Saturday", XmSTRING_DEFAULT_CHARSET),
  W_Saturday_Label= XtVaCreateManagedWidget("W_Saturday_Label",
					    xmLabelWidgetClass,
					    W_Calendar_Form2,
					    XmNlabelString,LabelString,
					    XmNalignment, XmALIGNMENT_BEGINNING,
					    XmNleftAttachment, XmATTACH_POSITION,
					    XmNleftPosition, 6,
					    XmNrightAttachment, XmATTACH_POSITION,
					    XmNrightPosition, 7,
					    XmNtopAttachment, XmATTACH_POSITION,
					    XmNtopPosition, 0,
					    XmNbottomAttachment, XmATTACH_POSITION,
					    XmNbottomPosition, 1,
					    NULL,0);
  XmStringFree(LabelString);


  for (row=1,position=0,lcv = 0;lcv < 42;lcv++,position++)
    {
      LabelString = XmStringCreate("", XmSTRING_DEFAULT_CHARSET);
/*       fprintf(stderr,"%d %d %d %d\n",lcv,lcv%7,position,row); */
      if (lcv%7 == 0)
	{
	  position = 0;
	  row++;
	}
      TextSelection_Array[0] =  XmSELECT_POSITION;
      W_Days_Button[lcv] = XtVaCreateManagedWidget("W_Saturday_Button",
						   xmTextFieldWidgetClass,
						   W_Calendar_Form2,
						   XmNlabelString,LabelString,
						   XmNalignment, XmALIGNMENT_BEGINNING,
						   XmNshadowThickness,0,
						   XmNleftAttachment, XmATTACH_POSITION,
						   XmNleftPosition, position,
						   XmNrightAttachment, XmATTACH_POSITION,
						   XmNrightPosition, position+1,
						   XmNtopAttachment, XmATTACH_POSITION,
						   XmNtopPosition, row,
						   XmNbottomAttachment, XmATTACH_POSITION,
						   XmNbottomPosition, row+1,
						   XmNorientation, XmHORIZONTAL,
						   XmNeditable, False,
						   XmNcursorPositionVisible, False,
						   XmNcolumns,5,
						   XmNselectionArray,TextSelection_Array,
						   XmNselectionArrayCount,1,
						   NULL,0);
      XmStringFree(LabelString);
      sprintf(Day," %d",lcv);
/*       printf("%s %d\n",Day,lcv); */
      XtAddEventHandler(W_Days_Button[lcv],ButtonPressMask,False,
			(XtEventHandler) DisplayDaily_Appointments, (XtPointer) lcv);
    }
  
  W_Calendar_Separator = XtVaCreateManagedWidget("W_Calendar_Separator",
						 xmSeparatorWidgetClass, 
						 W_Calendar_Form,
						 XmNleftAttachment, XmATTACH_FORM,
						 XmNrightAttachment, XmATTACH_FORM,
						 XmNtopAttachment, XmATTACH_WIDGET,
 						 XmNtopWidget, W_Calendar_Form2,
						 XmNorientation, XmHORIZONTAL,
						 NULL, 0);

  LabelString = XmStringCreate("Help: ", XmSTRING_DEFAULT_CHARSET),
  W_Help_Label= XtVaCreateManagedWidget("W_Help_Label",
					xmLabelWidgetClass,
					W_Calendar_Form,
					XmNlabelString,LabelString,
					XmNleftAttachment, XmATTACH_FORM,
					XmNleftOffset, 5,
					XmNrightAttachment, XmATTACH_FORM,
					XmNrightOffset, 5,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNbottomOffset, 5,
					XmNtopAttachment, XmATTACH_WIDGET,
					XmNtopWidget, W_Calendar_Separator,
					XmNtopOffset, 5,
					XmNalignment, XmALIGNMENT_BEGINNING,
					XmNrecomputeSize, False,
					NULL,0);
  XmStringFree(LabelString);
}


void Create_ButtonBar()
{
  XmString LabelString;
  Widget W_ButtonBar_Form;
  Widget W_ButtonBar_RC;
  Widget W_Next_Button;
  Widget W_Prev_Button;
  Widget W_Today_Button;
  Widget W_Goto_Button;
  Widget W_Julian_Button;

  W_ButtonBar_Frame = XtVaCreateManagedWidget("W_ButtonBar_Frame",
					      xmFrameWidgetClass, 
					      W_Main_Form,
					      XmNleftAttachment, XmATTACH_FORM,
					      XmNrightAttachment, XmATTACH_FORM,
					      XmNtopAttachment, XmATTACH_WIDGET,
					      XmNtopWidget, W_MenuBar,
					      NULL,0);
  
  W_ButtonBar_Form = XtVaCreateManagedWidget("W_ButtonBar_Form",
					     xmFormWidgetClass, 
					     W_ButtonBar_Frame,
					     NULL,0);
  
  W_ButtonBar_RC = XtVaCreateManagedWidget("W_ButtonBar_RC",
					   xmRowColumnWidgetClass, 
					   W_ButtonBar_Form,
					   XmNleftAttachment, XmATTACH_FORM,
					   XmNrightAttachment, XmATTACH_FORM,
					   XmNtopAttachment, XmATTACH_FORM,
					   XmNorientation, XmHORIZONTAL,
					   XmNpacking, XmPACK_COLUMN,
					   XmNentryAlignment, XmALIGNMENT_CENTER,
					   NULL,0);

  LabelString = XmStringCreate("Previous", XmSTRING_DEFAULT_CHARSET),
  W_Prev_Button= XtVaCreateManagedWidget("W_Prev_Button",
					 xmArrowButtonWidgetClass,
					 W_ButtonBar_RC,
					 XmNlabelString,LabelString,
					 XmNarrowDirection, XmARROW_LEFT,
					 NULL,0);
  XmStringFree(LabelString);
  XtAddCallback(W_Prev_Button, XmNactivateCallback,  
		(XtCallbackProc)  PreviousMonth_Calendar, NULL);
  XtAddEventHandler(W_Prev_Button,EnterWindowMask,False,
		    (XtEventHandler) Help_Line, (XtPointer) "prev");
  
  
   LabelString = XmStringCreate("Next", XmSTRING_DEFAULT_CHARSET),
   W_Next_Button= XtVaCreateManagedWidget("W_Next_Button",
					  xmArrowButtonWidgetClass,
					  W_ButtonBar_RC,
					  XmNlabelString,LabelString,
					  XmNarrowDirection, XmARROW_RIGHT,
					  NULL,0);
   XmStringFree(LabelString);
   XtAddCallback(W_Next_Button, XmNactivateCallback,  
		 (XtCallbackProc)  NextMonth_Calendar, NULL);
   XtAddEventHandler(W_Next_Button,EnterWindowMask,False,(XtEventHandler) Help_Line, 
		     (XtPointer) "next");

   LabelString = XmStringCreate("Today", XmSTRING_DEFAULT_CHARSET),
   W_Today_Button= XtVaCreateManagedWidget("W_Today_Button",
					  xmPushButtonWidgetClass,
					 W_ButtonBar_RC,
					 XmNlabelString,LabelString,
					 NULL,0);
  XmStringFree(LabelString);

  XtAddCallback(W_Today_Button, XmNactivateCallback,  
		(XtCallbackProc)  Show_Today, NULL);

  XtAddEventHandler(W_Today_Button,EnterWindowMask,False,(XtEventHandler) Help_Line, 
		    (XtPointer) "today");

  LabelString = XmStringCreate("Goto", XmSTRING_DEFAULT_CHARSET),
  W_Goto_Button= XtVaCreateManagedWidget("W_Goto_Button",
					 xmPushButtonWidgetClass,
					 W_ButtonBar_RC,
					 XmNlabelString,LabelString,
					 NULL,0);
  XmStringFree(LabelString);

  XtAddCallback(W_Goto_Button, XmNactivateCallback,  
		(XtCallbackProc)  Goto_Form, NULL);

  XtAddEventHandler(W_Goto_Button,EnterWindowMask,False,(XtEventHandler) Help_Line, 
		    (XtPointer) "goto");

  LabelString = XmStringCreate("Use Julian Date ", XmSTRING_DEFAULT_CHARSET),
  W_Julian_Button= XtVaCreateManagedWidget("W_Julian_Button",
					   xmToggleButtonWidgetClass,
					   W_ButtonBar_RC,
					   XmNlabelString,LabelString,
					   NULL,0);
  XmStringFree(LabelString);
  
  XtAddCallback(W_Julian_Button, XmNvalueChangedCallback,  
		(XtCallbackProc)  ToggleJulianDates, NULL);
  
  XtAddEventHandler(W_Julian_Button,EnterWindowMask,False,(XtEventHandler) Help_Line, 
		    (XtPointer) "julian");

}

void  Help_Line(Widget   w,
		char     *Key)
{
  XmString LabelString;
  char LabelBuffer[MY_BUFSIZ];
  int lcv;

/*   fprintf(stderr,"Help: %s\n",Key); */
  lcv = 0;
  while (Help_List[lcv].Key != NULL)
    {
      if (strcmp(Help_List[lcv].Key,Key) == 0)
	{
	  sprintf(LabelBuffer,"Help: %s",Help_List[lcv].Value);
	  LabelString = XmStringCreate(LabelBuffer, XmSTRING_DEFAULT_CHARSET),
	  XtVaSetValues(W_Help_Label,
			XmNlabelString,LabelString,
			NULL,0);
	  XmStringFree(LabelString);
	  return;
	}
      lcv++;
    }
}

void Print_Usage(char *ProgramName)
{
  fprintf(stderr,"Usage: %s [options] [standard X options]\n\nOptions:\n",ProgramName);
  fprintf(stderr,"\t-help or -h     -- What you see here.\n");
  fprintf(stderr,"\t-user or -u     -- Another user's calendar\n");
}

void Show_Today()

{
  Set_Today();
  G_Current_Year = G_Today_Year;
  G_Current_Month = G_Today_Month;
  DisplayMonth_Calendar(G_Current_Month,G_Current_Year);
  if (G_DisplayYearlyViewVisible == True)
    {
      DrawYearCalendar();
    }
}

void NextMonth_Calendar()
{
  G_Current_Day = 1;
  G_Current_Month++;
  if (G_Current_Month > 12)
    {
      G_Current_Month = 1;
      G_Current_Year++;
      if (G_Current_Year > 9999)
	{
	  G_Current_Year = 1;
	}
    }
  DisplayMonth_Calendar(G_Current_Month,G_Current_Year);
  if (G_DisplayYearlyViewVisible == True)
    {
      DrawYearCalendar();
    }
  if (G_AppointmentFormVisible == True)
    {
      CreateAppointment_Form(G_Current_Day,G_Current_Month,G_Current_Year);
    }
}


void PreviousMonth_Calendar()
{
  G_Current_Day = 1;
  G_Current_Month--;
  if (G_Current_Month < 1)
    {
      G_Current_Month = 12;
      G_Current_Year--;
      if (G_Current_Year < 1)
	{
	  G_Current_Year = 9999;
	}
    }
  DisplayMonth_Calendar(G_Current_Month,G_Current_Year);
  if (G_DisplayYearlyViewVisible == True)
    {
      DrawYearCalendar();
    }
  if (G_AppointmentFormVisible == True)
    {
      CreateAppointment_Form(G_Current_Day,G_Current_Month,G_Current_Year);
    }
}



void DisplayMonth_Calendar(int Month,
			   int Year)
{
  XmString LabelString;
  char LabelBuffer[BUFSIZ];
  int days[42];
  int jdays[42];
  int lcv;
  int Thickness = 1;

  Set_Today();
  sprintf(LabelBuffer,"%s",G_MonthNames[Month-1]);
  LabelString = XmStringCreate(LabelBuffer, XmSTRING_DEFAULT_CHARSET);
  XtVaSetValues(W_Month_Label,
		XmNlabelString,LabelString,
		NULL,0);
  XmStringFree(LabelString);
  sprintf(LabelBuffer,"%d",Year);
  LabelString = XmStringCreate(LabelBuffer, XmSTRING_DEFAULT_CHARSET);
  XtVaSetValues(W_Year_Label,
		XmNlabelString,LabelString,
		NULL,0);
  XmStringFree(LabelString);
  day_array(Month, Year, days);
  day_array(Month, Year, jdays);
  for (lcv = 0; lcv < 42 ; lcv ++)
    {
/*       fprintf(stderr,"%d\n",lcv); */
      if (days[lcv] != -1)
	{
	  if (G_Julian == 1)
	    {
	      sprintf(LabelBuffer,"%d",jdays[lcv]);
	    }
	  else {
	    sprintf(LabelBuffer,"%d",days[lcv]);
	  }
	  if ((days[lcv] == G_Today_Day) && (G_Current_Month == G_Today_Month) 
	      && (G_Current_Year == G_Today_Year))
	    {
	      XtVaSetValues(W_Days_Button[lcv],
			    XmNshadowThickness, 1,
			    NULL,0);
	    }
	  else {
	    XtVaSetValues(W_Days_Button[lcv],
			  XmNshadowThickness, 0,
			  NULL,0);
	  }
	  XmTextFieldSetString(W_Days_Button[lcv],LabelBuffer);
	  XtSetSensitive(W_Days_Button[lcv],True);
	}
      else {
	XmTextFieldSetString(W_Days_Button[lcv],"");
	XtSetSensitive(W_Days_Button[lcv],False);
      }
    }  
  if ( (Month == G_Today_Month) &&
      (Year == G_Today_Year) &&
      (days[lcv] == G_Today_Day))
    {
      Thickness = 3;
    }
  else {
    Thickness = 1;
  }
}



void DisplayDaily_Appointments(Widget   w,
			       int     Day)
{
/*   static Count = 0; */
/*   static int LastDay = -1; */
  
  G_Current_Day = atoi(XmTextFieldGetString(W_Days_Button[Day]));

  CreateAppointment_Form(G_Current_Day,G_Current_Month,G_Current_Year);
/*  Count++;
  if (Count == 2)
    {
      if (Day == LastDay)
 	{ 
	  fprintf(stderr,"2nd click DAY = %d\n",Day);
	  CreateAppointment_Form(G_Current_Day,G_Current_Month,G_Current_Year);
 	} 
      Count = 0;
      LastDay = -1;
    }
  else {
    fprintf(stderr,"1st click DAY = %d\n",Day);
    LastDay = Day;
  }
*/
}

void ToggleJulianDates(Widget w)
{
  if (XmToggleButtonGetState(w) == True)
    {
      G_Julian = 1;
    }
  else {
    G_Julian = 0;
  }
  DisplayMonth_Calendar(G_Current_Month,G_Current_Year);
  if (G_DisplayYearlyViewVisible == True)
    {
      DrawYearCalendar();
    }
}


void Set_Today()
{
  static char  Time_Str[10];
  time_t Time_Enc;
  struct tm *Time_Struct;

  time(&Time_Enc);
  Time_Struct = localtime(&Time_Enc);
  strftime(Time_Str,10,"%d",Time_Struct);
  G_Today_Day = atoi(Time_Str);
  strftime(Time_Str,10,"%m",Time_Struct);
  G_Today_Month = atoi(Time_Str);
  strftime(Time_Str,10,"%Y",Time_Struct);
  G_Today_Year = atoi(Time_Str);
}


void  Set_Calendar_Filename()
{
  if (G_Calendar_Filename != NULL)
    {
      XmTextFieldSetString(W_Filename_Text,G_Calendar_Filename);
    }
}
