#define MAIN_H

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <pwd.h>

#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/cursorfont.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/DrawingA.h>
#include <Xm/CascadeBG.h>
#include <Xm/ArrowB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrolledW.h>
#include <Xm/Form.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/Separator.h>
#include <Xm/Frame.h>
#include <Xm/BulletinB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/FileSB.h>
#include <Xm/MessageB.h>

#include "general.h"

/*******************

  User-Customizable Defines

********************/

#ifdef HPUX
#define DEFAULT_PRINT_COMMAND     "lpr -dps1 "
#else
#define DEFAULT_PRINT_COMMAND     "/usr/ucb/lpr "
#endif

#define DEFAULT_PRINT_FILENAME    "xmcalendar.ps"
#define DEFAULT_CALENDAR_FILENAME  ".xmcalendar.dat"  

#if !defined(HELP_FILE)
#define HELP_FILE                 "xmcalendar.hlp"
#endif



/*******************

   Defines

********************/

#define TEMP_FILENAME_EXTENSION   ".tmp"
#define PROGRAM_TITLE             "XmCalendar Version 1.0"
#define PROGRAM_DESCRIPTION       "XmCalendar\n\nThe Motif Calendar\n\nVersion 1.0 \n\nCopyright 1994 by Boyd Fletcher\n\nSend problems to: boyd@ccpo.odu.edu"
#define RETURN_NOTOK 0
#define RETURN_OK    1
#define MY_BUFSIZ        1024
#define MY_BIG_BUFSIZ    4096

#define REPEAT_NONE      0
#define REPEAT_DAILY     1
#define REPEAT_WEEKLY    2
#define REPEAT_BIWEEKLY  3
#define REPEAT_MONTHLY   4
#define REPEAT_YEARLY    5

#define REMINDER_WARNING   10  /* minutes */

/******************

   Types

*******************/


typedef int ReturnType;

typedef struct _menu_item {
    char        *label;         /* the label for the item */
    WidgetClass *class;         /* pushbutton, label, separator... */
    char         mnemonic;      /* mnemonic; NULL if none */
    char        *accelerator;   /* accelerator; NULL if none */
    char        *accel_text;    /* to be converted to compound string */
    void       (*callback)();   /* routine to call; NULL if none */
    XtPointer    callback_data; /* client_data for callback() */
    struct _menu_item *subitems; /* pullright menu items, if not NULL */
} MenuItem;

typedef struct Appointment_Struct {
  int Minutes;
  int Hour;
  char *Message;
  int ListPosition;
  Bool ReminderAlarm;  /* true if to sound alarm */
  int ReminderWarning; /* in minutes */
  int Repeat;  /* see define above */
  int RepeatForNumDays; 
  struct Appointment_Struct *Next;
} Appointment_Type;


typedef struct Calendar_Struct {
  int Day;
  int Month;
  int Year;
  int NumberOfAppointments;
  Appointment_Type *Appointments;
  struct Calendar_Struct *Next;
} Calendar_Type;

/*

   Global Variables

*/

#include "prototypes.h"

#ifndef MAIN
extern XmFontList G_MainFontList;
extern XFontStruct *G_MainFontStruct;
extern XmFontList G_BigFontList;
extern XFontStruct *G_BigFontStruct;
extern XmFontList G_FixedFontList;
extern XFontStruct *G_FixedFontStruct;
extern char *G_PrinterName;
extern Calendar_Type *G_CalendarList;
extern int G_NumInCalendarList;
extern int G_CurrentNumInCalendarList;
extern Calendar_Type *G_CurrentEntryInCalendarList;
extern char *G_HelpString;
extern char *G_Calendar_Filename;
extern XtAppContext G_ApplicationContext;
extern Widget W_TopLevel;
extern Widget W_Main_Window, W_Main_Form, W_Main_Frame;
extern Widget W_MenuBar;
extern Bool G_ChangedNotSaved;
extern Bool G_OpenCalendar;
extern int G_Current_Year;
extern int G_Current_Month;
extern int G_Current_Day;
extern int G_Today_Year;
extern int G_Today_Month;
extern int G_Today_Day;
extern char *G_MonthNames[12];
extern Bool G_AppointmentFormVisible;
extern Bool G_DisplayYearlyViewVisible;
extern Bool G_ASCII_Mode;
extern int G_Julian;
#else
XmFontList G_MainFontList;
XFontStruct *G_MainFontStruct;
XmFontList G_BigFontList;
XFontStruct *G_BigFontStruct;
XmFontList G_FixedFontList;
XFontStruct *G_FixedFontStruct;
char *G_PrinterName = NULL;
Calendar_Type *G_CalendarList = NULL;
Calendar_Type *G_CurrentEntryInCalendarList = NULL;
int G_CurrentNumInCalendarList = 0;
int G_NumInCalendarList = 0;
char *G_HelpFile;
char *G_HelpString = NULL;
char *G_Calendar_Filename = NULL;
XtAppContext G_ApplicationContext;
Widget W_TopLevel;
Widget W_Main_Window, W_Main_Form, W_Main_Frame;
Widget W_MenuBar;
Bool G_ChangedNotSaved = False;
Bool G_OpenCalendar = False;
Bool G_AppointmentFormVisible = False;
int G_Current_Year = 0 ;
int G_Current_Month = 0;
int G_Current_Day = 0 ;
int G_Today_Year = 0 ;
int G_Today_Month = 0;
int G_Today_Day = 0 ;
int G_Julian = 0;
Bool G_DisplayYearlyViewVisible = False;
char *G_MonthNames[12] = {
  "January", "February", "March", "April", "May", "June",
  "July", "August", "September", "October", "November", "December" };
Bool G_ASCII_Mode = False;
#endif


